(* DESCRIPTION = MONDE SANS BSP.                             *)
(* CHARGEMENT DE LA VERSION TEXTUELLE D'UNE DESCRIPTION D'UN *)
(* MONDE. CES DONNEES SORTENT DIRECTEMENT DU CREATEUR DE     *)
(* NIVEAU.                                                   *)

open Vertexsbase ;;
open Linedefsbase ;;
open Sectorsbase ;;
open Boolextra ;;



let aDescrLoadVertexes in_channel =
 let header = (input_line in_channel) in
  if header <> "VERTEXES"
   then
    raise (Invalid_argument "Bad file type - No vertexes header")
   else
    begin
    let nbvexterxes = int_of_string (input_line in_channel) in
     for i=0 to nbvexterxes-1
      do
      let xv = float_of_string (input_line in_channel) in
      let yv = float_of_string (input_line in_channel) in
       addVertex { x = xv ; y = yv }
      done
    end
;;


let aDescrLoadLinedefs in_channel =
 let header = (input_line in_channel) in
  if header <> "LINEDEFS"
   then
    raise (Invalid_argument "Bad file type - No linedef header")
   else
    begin
    let nbLinedefs = int_of_string (input_line in_channel) in
     for i=0 to nbLinedefs-1
      do
      let lsrc = int_of_string (input_line in_channel) in
      let ldest = int_of_string (input_line in_channel) in
      let lsector = int_of_string (input_line in_channel) in
      let lnext = int_of_string (input_line in_channel) in
      let langle = float_of_string (input_line in_channel) in
      let llen = float_of_string (input_line in_channel) in
      let ladjacentsector = (match input_line in_channel with
         "NONE" -> -1
       | other -> int_of_string other) in
      let lmaintx = int_of_string (input_line in_channel) in
      let luppertx = int_of_string (input_line in_channel) in
      let llowertx = int_of_string (input_line in_channel) in
      let lpassable = bool_of_string (input_line in_channel) in
       addLinedef { src = lsrc ;
                    dest = ldest ;
                    sector = lsector ;
                    next = lnext ;
                    angle = langle ;
                    len = llen ;
                    adjacentsector = ladjacentsector ;
                    maintx = lmaintx ;
                    uppertx = luppertx ;
                    lowertx = llowertx ;
                    passable = lpassable }
      done
    end
;;



let aDescrLoadSectors in_channel =
 let header = (input_line in_channel) in
  if header <> "SECTORS"
   then
    raise (Invalid_argument "Bad file type - No sectors header")
   else
    begin
    let nbSectors = int_of_string (input_line in_channel) in
     for i=0 to nbSectors-1
      do
      let nbbounds = (int_of_string (input_line in_channel))
      and sbounds = ref [] in
       for l=0 to nbbounds-1
        do
        sbounds := !sbounds @ [(int_of_string (input_line in_channel))] ;
        done ;
      let sceiling = float_of_string (input_line in_channel) in
      let sfloor = float_of_string (input_line in_channel) in
       addSector { bounds = !sbounds ; ceilingh = sceiling; floorh = sfloor }
      done
    end
;;



let aDescrLoadDescription filename =
 let in_channel = open_in filename in
  let header = input_line in_channel in
   if header <> "DESCRIPTION-FILE"
    then raise (Invalid_argument "Bad file type - No main header")
    else
     begin
     aDescrLoadVertexes in_channel ;
     aDescrLoadLinedefs in_channel ;
     aDescrLoadSectors in_channel ;
     close_in in_channel
     end
;;
