open Vertexsbase ;;
open Linedefsbase ;;
open Sectorsbase ;;
open Bsptreebase ;;


let aLevelSaveVertexes out_channel =
 output_string out_channel "VERTEXES\n" ;
 output_string out_channel (string_of_int !nbVertexes) ;
 output_string out_channel "\n" ;
 for i=0 to !nbVertexes-1
  do
  let vert = globalvertextable.(i) in
   output_string out_channel (string_of_float vert.x) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_float vert.y) ;
   output_string out_channel "\n" ;
  done
;;


let aLevelSaveLinedefs out_channel =
 output_string out_channel "LINEDEFS\n" ;
 output_string out_channel (string_of_int !nbLinedefs) ;
 output_string out_channel "\n" ;
 for i=0 to !nbLinedefs-1
  do
  let line = globallinedeftable.(i) in
   output_string out_channel (string_of_int line.src) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_int line.dest) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_int line.sector) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_int line.next) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_float line.angle) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_float line.len) ;
   output_string out_channel "\n" ;
   (match (line.adjacentsector) with
      -1 -> output_string out_channel "NONE"
    | i -> output_string out_channel (string_of_int i)) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_int line.maintx) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_int line.uppertx) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_int line.lowertx) ;
   output_string out_channel "\n" ;
   output_string out_channel (string_of_bool line.passable) ;
   output_string out_channel "\n"
  done
;;


let aLevelSaveSectors out_channel =
 output_string out_channel "SECTORS\n" ;
 output_string out_channel (string_of_int !nbSectors) ;
 output_string out_channel "\n" ;
 for i=0 to !nbSectors-1
  do
  let sect = globalsectortable.(i) in
   output_string out_channel (string_of_int (List.length sect.bounds)) ;
   output_string out_channel "\n" ;
   List.iter (fun i -> output_string out_channel (string_of_int i) ;
                     output_string out_channel "\n") sect.bounds ;
  output_string out_channel (string_of_float sect.ceilingh) ;
  output_string out_channel "\n" ;
  output_string out_channel (string_of_float sect.floorh) ;
  output_string out_channel "\n" ;
  done
;;


let aLevelSaveBsp out_channel tree =
 let rec recSaveBsp = function
    Node { partline=pl_id ; leftbbox=lbb ;
           left=l ; rightbbox=rbb ; right=r } ->
     output_string out_channel "NODE\n" ;
     output_string out_channel (string_of_int pl_id) ;
     output_string out_channel "\n" ;
     output_string out_channel "LEFTBB\n" ;
     (match lbb with
        None -> output_string out_channel "NONE\n"
      | Some (f1,f2,f3,f4) ->
         output_string out_channel "SOME\n" ;
         output_string out_channel (string_of_float f1) ;
         output_string out_channel "\n" ;
         output_string out_channel (string_of_float f2) ;
         output_string out_channel "\n" ;
         output_string out_channel (string_of_float f3) ;
         output_string out_channel "\n" ;
         output_string out_channel (string_of_float f4) ;
         output_string out_channel "\n") ;
     recSaveBsp l ;
     output_string out_channel "RIGHTBB\n" ;
     (match rbb with
        None -> output_string out_channel "NONE\n"
      | Some (f1,f2,f3,f4) ->
         output_string out_channel "SOME\n" ;
         output_string out_channel (string_of_float f1) ;
         output_string out_channel "\n" ;
         output_string out_channel (string_of_float f2) ;
         output_string out_channel "\n" ;
         output_string out_channel (string_of_float f3) ;
         output_string out_channel "\n" ;
         output_string out_channel (string_of_float f4) ;
         output_string out_channel "\n") ;
     recSaveBsp r ;
  | Leaf optl ->
     begin
     match optl with
        [] ->
         output_string out_channel "NULL\n"
      | line_id_lst ->
         output_string out_channel "LEAF\n" ;
         output_string out_channel (string_of_int (List.length line_id_lst)) ;
         output_string out_channel "\n" ;
         List.iter (fun i -> output_string out_channel (string_of_int i) ;
                           output_string out_channel " ") line_id_lst ;
         output_string out_channel "\n"
     end
  in
   output_string out_channel "BSP-PART\n" ;
   recSaveBsp tree
;;



let aLevelSaveLevel filename bsp_tree =
 let out_channel = open_out filename in
  begin
  output_string out_channel "LEVEL-FILE\n" ;
  aLevelSaveVertexes out_channel ;
  aLevelSaveLinedefs out_channel ;
  aLevelSaveSectors out_channel ;
  aLevelSaveBsp out_channel bsp_tree ;
  close_out out_channel
  end
;;

