open Bclinedefs ;;
open Floatextra ;;
open Linedefsbase ;;
open Vertexsbase ;;
open Bsptreebase ;;
open Listextra ;;



(* ****************************************************************** *)
(* FONCTION QUI CALCULE LE NOMBRE D'INTERSECTION DE LA LIGNE INFINIE  *)
(* ORIENTEE NUMERO partline_id AVEC TOUTES LES AUTRES LIGNES DONT LES *)
(* NUMEROS SONT DANS LA LISTE line_id_lst.                            *)
(* ****************************************************************** *)
let rec calcIntersectNumber partline_id line_id_lst =
 match line_id_lst with
    [] -> 0
  | line_id::q ->
     if line_id = partline_id
      then (* 0 + *) (calcIntersectNumber partline_id q)
      else if existIntersect partline_id line_id
            then 1 + (calcIntersectNumber partline_id q)
            else (* 0 + *) (calcIntersectNumber partline_id q)
;;


(* ******************************************************************** *)
(* FONCTION QUI DETERMINE LA MEILLEURE LIGNE DE PARTITION CAD CELLE QUI *)
(* MINIMISANT LE NOMBRE DE DECOUPAGE DE LIGNE.                          *)
(* int list -> int                                                      *)
(* ******************************************************************** *)
let findBestPartLine line_id_lst =
 match line_id_lst with
    [] ->
     (* SI LA LISTE ORIGINALE EST VIDE ALORS CA N'EST PAS NORMAL   *)
     (* CETTE FONCTION NE DOIT PAS ETRE APELLEE (NORMALEMENT) AVEC *)
     (* UNE LISTE VIDE DE NUMEROS LIGNES. SI ON ARRIVE ICI C'EST   *)
     (* QU'IL Y A UN PEPIN AVANT...                                *)
     raise (Invalid_argument "choosePartLine")
  | line_id::q ->
     (* ON INITIALISE NOS COMPTEURS TEMPORAIRES AVEC LE *)
     (* TRAITEMENT SUR LA PREMIERE LIGNE DE LA LISTE.   *)
     let bestline = ref line_id in
     let minintersec = ref (calcIntersectNumber line_id line_id_lst) in
      (* ET MAINTENANT ON FAIT LA RECHERCHE *)
      (* SUR LE RESTE DE LA LISTE...        *)
      for i = 1 to (List.length line_id_lst) - 1
       do
       let partline = List.nth line_id_lst i in
        let nbinter = calcIntersectNumber partline line_id_lst in
         if nbinter < !minintersec
          then (bestline := partline ; minintersec := nbinter)
       done ;
       !bestline
;;



(* ************************************************************ *)
(* FONCTION QUI SE CHARGE DE FAIRE LES DECOUPAGES DE LINEDEFS   *)
(* CONTENUES DANS LA LISTE line_id_lst ET LES RAJOUTS DE        *)
(* VERTEXES >>PAR EFFET DE BORD<<, EN FONCTION DE LA LIGNE DE   *)
(* PARTITION NUMERO partline_id.                                *)
(* >>ATTENTION<< : LA LIGNE DE PARTITION NE DOIT PAS SE TROUVER *)
(* PARMIS LES LIGNES SE TROUVANT DANS LA LISTE line_id_lst.     *)
(* ELLE RETOURNE LA LISTE DES LIGNES EVENTUELLEMENT DECOUPEES   *)
(* ISSUES DU TRAITEMENT DE LA LISTE ORIGINALE line_id_lst.      *)
(* ************************************************************ *)
let rec splitLines partline_id line_id_lst =
 match line_id_lst with
    [] -> []
  | line_id::q ->
     (* ON REGARDE S'IL EXISTE UNE INTERSECTION *)
     if existIntersect partline_id line_id
      then
       begin
       (* ON RECUPERE LE POINT D'INTERSECTION *)
       let (xi,yi) = findIntersect partline_id line_id in
        (* SI LE POINT D'INTERSECTION CORRESPOND A UN DES VERTEXES     *)
        (* EXTREMITE DU SEGMENT ALOPRS ON NE LE CONSIDERE PAS VRAIMENT *)
        (* COMME UNE INTERSECTION CAR IL N'Y A PAS A PROPREMENT PARLER *)
        (* BESOIN DE DECOUPAGE DE LA LIGNE...                          *)
        let hstart = findLinedefStart line_id
        and hend = findLinedefEnd line_id in
         if ((float_eq (fst hstart) xi) && (float_eq (snd hstart) yi))
                                        ||
              ((float_eq (fst hend) xi) && (float_eq (snd hend) yi))
          then
           (* EN FAIT C'ETAIT UNE "FAUSSE" INTERSECTION, DONC ON LAISSE *)
           line_id :: (splitLines partline_id q)
          else
           begin
           (* C'ETAIT EN FAIT UNE VERITABLE INTERSECTION *)
           let line = globallinedeftable.(line_id) in
            (* ON CREE DONC UN VERTEX AU POINT D'INTERSECTION *)
            let newvertexnumber = addVertex { x = xi ; y = yi } in
             (* ON CREE UNE NOUVELLE LIGNE (SEG). C'EST CELLE QUI    *)
             (* AURA POUR DESTINATION L'ANCIENNE DESTINATION ET POUR *)
             (* ORIGINE LE VERTEX FRAICHEMENT CREE.                  *)
             let newsegnumber = addLinedef { src = newvertexnumber ;
                                             dest = line.dest ;
                                             sector = line.sector ;
                                             next = line.next ;
                                             angle = getLineAngle newvertexnumber line.dest ;
                                             len = getLineLen newvertexnumber line.dest ;
                                             adjacentsector = line.adjacentsector ;
                                             maintx = line.maintx ;
                                             uppertx = line.uppertx ;
                                             lowertx = line.lowertx ;
                                             passable = line.passable } in
              (* ON MODIFIE EN PLACE LA LIGNE D'ANTANT POUR QU'ELLE SERVE   *)
              (* DE SEG DONT ON GARDE LE POINT DE DEPART. SON NOUVEAU POINT *)
              (* D'ARRIVEE EST DONC LE POINT D'INTERSECTION.                *)
              globallinedeftable.(line_id) <-
                                   { src = line.src ;
                                     dest = newvertexnumber ;
                                     sector = line.sector ;
                                     next = newsegnumber ;
                                     angle = getLineAngle line.src newvertexnumber ;
                                     len = getLineLen line.src newvertexnumber ;
                                     adjacentsector = line.adjacentsector ;
                                     maintx = line.maintx ;
                                     uppertx = line.uppertx ;
                                     lowertx = line.lowertx ;
                                     passable = line.passable } ;
              (* IL FAUT MAINTENANT RETOURNER LA LISTE DE DEUX SEGMENTS, *)
              (* DONT LE NOUVEAU SUIVIE DU RESULTAT DU TRAITEMENT DE LA  *)
              (* SUITE...                                                *)
              line_id :: newsegnumber :: (splitLines partline_id q)
           end
       end
     else
      (* IL N'EXISTE PAS D'INTERSECTION DONC ON VA VOIR LA SUITE *)
      (* EN RETOURNANT NOTRE LIGNE INCHANGEE SUIVIE DE L'APPEL   *)
      (* RECURSIF TRAITANT LA SUITE DE NOTRE LISTE.              *)
      line_id :: (splitLines partline_id q)
;;


(* CALCULE LA BOUDING BOX AFFERENTE A UNE LINEDEF *)
let compute_bounding_line line_id =
 let (x1,y1) = findLinedefStart line_id
 and (x2,y2) = findLinedefEnd line_id in
  ((min x1 x2), (min y1 y2), (max x1 x2), (max y1 y2))
;;


(* ******************************************************** *)
(* FONCTION QUI CALCULE LA BOUNDING BOX ASSOCIEE A UN ARBRE *)
(* ELLE RETOURNE UN QUARTET DANS L'ORDRE SUIVANT:           *)
(* (xmin,ymin,xmax,ymax).                                   *)
(* CETTE BOUNDING EST CALCULEE A PARTIE DES BOUNDING BOXES  *)
(* ASSOCIEES AU SOUS ARBRE DE GAUCHE ET DE DROITE A CE      *)
(* NOEUD. AUTREMENT DIT, LA BB CALCULEE POUR CE NOEUD NE    *)
(* PAS STOCKEE A CE NOEUD, MAIS DANS SON PARENT AU NIVEAU   *)
(* DROIT OU GAUCHE, SELON QUE NE NOEUD ACTUEL EST LE FILS   *)
(* DROITE OU DE GAUCHE DE SON PARENT.                       *)
(* ******************************************************** *)
let compute_bounding_node n =
 match n.leftbbox with
    None ->
     begin
     match n.rightbbox with
        None ->
         raise (Invalid_argument "Ill-formed tree")
      | Some (rightxmin,rightymin,rightxmax,rightymax) ->
         let (partxmin,partymin,partxmax,partymax) = compute_bounding_line n.partline
          in Some ((min rightxmin partxmin),
                   (min rightymin partymin),
                   (max rightxmax partxmax),
                   (max rightymax partymax))
     end
  | Some (leftxmin,leftymin,leftxmax,leftymax) ->
     begin
     match n.rightbbox with
        None ->
         let (partxmin,partymin,partxmax,partymax) = compute_bounding_line n.partline
          in Some ((min leftxmin partxmin),
                   (min leftymin partymin),
                   (max leftxmax partxmax),
                   (max leftymax partymax))
      | Some (rightxmin,rightymin,rightxmax,rightymax) ->
         let (partxmin,partymin,partxmax,partymax) = compute_bounding_line n.partline
         and (tmp1,tmp2,tmp3,tmp4) = (min leftxmin rightxmin,
                                      min leftymin rightymin,
                                      max leftxmax rightxmax,
                                      max leftymax rightymax) in
          Some ((min partxmin tmp1),
                (min partymin tmp2),
                (max partxmax tmp3),
                (max partymax tmp4))
     end
;;



(* CALCULE LA BOUDING BOX AFFERENTE A UNE LISTE DE LINEDEFS *)
let compute_bounding_linelst = function
   [] -> raise (Invalid_argument "compute_bounding_linelst")
 | line_id::queue ->
    let (x1,y1,x2,y2) = compute_bounding_line line_id in
     let minx = ref x1
     and miny = ref y1
     and maxx = ref x2
     and maxy = ref y2 in
     let rec reccompute = function
        [] -> (!minx, !miny, !maxx, !maxy)
      | h::q ->
        let (x1,y1,x2,y2) = compute_bounding_line h in
         minx := min !minx x1 ;
         miny := min !miny y1 ;
         maxx := max !maxx x2 ;
         maxy := max !maxy y2 ;
         reccompute q
      in reccompute queue
;;



(* CALCULE LA BOUNDING BOX POUR UN ARBRE *)
let compute_bounding_box = function
   Leaf lopt ->
     begin
     match lopt with
       [] -> None
     | line_id_lst -> Some (compute_bounding_linelst line_id_lst)
     end
 | Node n -> compute_bounding_node n
;;



(* **************************************************************** *)
(* FONCTION QUI CONSTRUIT LE BSP A PARTIR D'UNE LISTE DE NUMEROS DE *)
(* LINEDEFS line_id_lst ET QUI RENVOIE CE BSP.                      *)
(* **************************************************************** *)
let rec buildTree line_id_lst =
 (* SI LA LISTE FORME UN POLYGONE CONVEXE ET QUE TOUTES LES   *)
 (* LINEDEFS APPARTIENNENT AU MEME SECTEUR, ALORS ON S'ARRETE *)
 if ((isConvexPolygon line_id_lst line_id_lst) && (isSingleSectorLines line_id_lst))
  then Leaf line_id_lst
  else
   (* SINON, ON RECURSE... *)
   begin
   (* ON CHOISIT UNE LIGNE DE PARTITION PARMIS *)
   (* LES LIGNES PRESENTES DANS NOTRE LISTE.   *)
   let partline_id = findBestPartLine line_id_lst in
    (* ON VIRE CETTE LIGNE DE LA LISTE *)
    let new_line_id_lst = except partline_id line_id_lst in
     (* ON CHERCHE A RECUPERER LA LISTE DES LIGNES EVENTUELLEMENT *)
     (* SPLITEES, EN SACHANT QUE PAR EFFET DE BORD, LES TABLES DE *)
     (* VERTEXES ET DE LINEDEFS ONT ETE MISES A JOUR              *)
     let new_line_id_lst2 = splitLines partline_id new_line_id_lst in
      (* ON RECURSE A GAUCHE ET A DROITE... *)
      let (left_l, right_l) = partitionList partline_id new_line_id_lst2 in
       let left_tr = buildTree left_l
       and right_tr = buildTree right_l in
        let left_bounding_box = compute_bounding_box left_tr
        and right_bound_box = compute_bounding_box right_tr
         in Node ({ partline = partline_id ;
                 leftbbox = left_bounding_box ;
                 left = left_tr ;
                 rightbbox = right_bound_box ;
                 right = right_tr })
   end
;;

