open Linedefsbase ;;
open Vertexsbase ;;
open Floatextra ;;
open Angleextra ;;
open Location ;;



(* ***************************************************************** *)
(* RENVOIE LA LISTE DES LIGNES CONTENUES DANS LE VECTEUR DE LINEDEFS *)
(* ***************************************************************** *)
let makeListOfLinedefsVect () =
 let line_id_lst = ref [] in
  for i = 0 to (!nbLinedefs) - 1
   do
   line_id_lst := (!line_id_lst) @ [i]
   done ;
   !line_id_lst
;;


(* ************************************************************ *)
(* FONCTION QUI RENVOIR LA PAIRE DE COORDONNEES DE DEPART DE LA *)
(* LIGNE NUMERO n.                                              *)
(* int -> (float * float)                                       *)
(* ************************************************************ *)
let findLinedefStart n =
 let line = globallinedeftable.(n) in
  let vertexstart = globalvertextable.(line.src) in
   (vertexstart.x, vertexstart.y)
;;


(* *************************************************************** *)
(* FONCTION QUI RENVOIR LA PAIRE DE COORDONNEES DE FIN DE LA LIGNE *)
(* NUMERO n.                                                       *)
(* int -> (float * float)                                       *)
(* *************************************************************** *)
let findLinedefEnd n =
 let line = globallinedeftable.(n) in
  let vertexend = globalvertextable.(line.dest) in
   (vertexend.x, vertexend.y)
;;


(* ****************************************************************** *)
(* FONCTION POUR SAVOIR SI DEUX LIGNES S'INTERSECTENT                 *)
(* DONT LA PREMIERE EST LA LIGNE DE PARTITION ET LA SECONDE LE        *)
(* SEGMENT. L'INTERSECTION DOIT SE PRODUIRE ABSOLUMENT DANS LES       *)
(* LIMITES DU SEGMENT, PAR CONTRE N'IMPORTE OU LE LONG DE LA LIGNE DE *)
(* PARTITIONNEMENT.                                                   *)
(* SI LES 2 DROITES SONT CONFONDUES ALORS ON CONSIDERE QU'IL N'Y A    *)
(* PAS D'INTERSECTION.                                                *)
(* ****************************************************************** *)
let existIntersect linepart_id otherline_id =
 let (xa,ya) = findLinedefStart linepart_id in
 let (xb,yb) = findLinedefEnd linepart_id in
 let (xc,yc) = findLinedefStart otherline_id in
 let (xd,yd) = findLinedefEnd otherline_id in
  let denom = (xb-.xa) *. (yd-.yc) -. (yb-.ya) *. (xd-.xc) in
   if float_eq denom 0.0
    then false
    else
     begin
     (* IL FAUT REGARDER SI L'INTERSECTION A BIEN LIEU *)
     (* ENTRE LES EXTREMITES DU SECOND SEGMENT. *)
     let s = ((ya-.yc) *. (xb-.xa) -. (xa-.xc) *. (yb-.ya)) /. denom in
      (s <= 1.0 && s >= 0.0)
     end
;;


(* ****************************************************************** *)
(* FONCTION POUR CALCULER LE POINT D'INTERSECTION DE 2 D'UNE LIGNE DE *)
(* PARTITIONNEMENT ORIENTEE ET D'UN SEGMENT ORIENTE.                  *)
(* ATTENTION : NE VERIFIE PAS SI CE POINT EXISTE.                     *)
(* ****************************************************************** *)
let findIntersect linepart_id otherline_id =
 let (xa,ya) = findLinedefStart linepart_id in
 let (xb,yb) = findLinedefEnd linepart_id in
 let (xc,yc) = findLinedefStart otherline_id in
 let (xd,yd) = findLinedefEnd otherline_id in
  let denom = (xb-.xa) *. (yd-.yc) -. (yb-.ya) *. (xd-.xc) in
   let r = ((ya-.yc) *. (xd-.xc) -. (xa-.xc) *. (yd-.yc)) /. denom
   and s = ((ya-.yc) *. (xb-.xa) -. (xa-.xc) *. (yb-.ya)) /. denom in
    (xa+.r*.(xb-.xa), ya+.r*.(yb-.ya))
;;


(* ***************************************************** *)
(* CALCULE L'ANGLE D'UNE DROITE AVEC L'AXE DES ABSCISSES *)
(* ***************************************************** *)
let getLineAngle v1 v2 =
 let x1 = (globalvertextable.(v1)).x
 and y1 = (globalvertextable.(v1)).y
 and x2 = (globalvertextable.(v2)).x
 and y2 = (globalvertextable.(v2)).y in
  if (x2 <> x1) then
   atan ((y2-.y1)/.(x2-.x1))
  else
   if (y2 < y1) then toRad (-90.0) else toRad 90.0
;;


(* *************************************** *)
(* CALCULE LA DISTANCE ENTRE DEUX VERTICES *)
(* *************************************** *)
let getLineLen v1 v2 =
 let x1 = (globalvertextable.(v1)).x
 and y1 = (globalvertextable.(v1)).y
 and x2 = (globalvertextable.(v2)).x
 and y2 = (globalvertextable.(v2)).y in
  sqrt ((x2-.x1)*.(x2-.x1) +. (y2-.y1)*.(y2-.y1))
;;



(* ****************************************************************** *)
(* RETOURNE LE NUMERO DE LA LIGNE QUI SUIT LA LIGNE n DANS LE SECTEUR *)
(* ****************************************************************** *)
let findLinedefNext n =
 let line = globallinedeftable.(n) in
  line.next
;;


(* ************************************************************* *)
(* FONCTION QUI DIT SI UNE LISTE DE NUMEROS DE LIGNES FORMENT UN *)
(* POLYGONE CONVEXE.                                             *)
(* EN FAIT, ELLE VERIFIE QUE CHAQUE LIGNE SE TROUVE A DROITE DE  *)
(* CELLE QUI LA PRECEDE DANS LE SECTEUR SI CELLE-CI SE TROUVE    *)
(* DANS LA LISTE.                                                *)
(* int list -> bool                                              *)
(* ************************************************************* *)
let rec isConvexPolygon original_list line_id_lst =
(* print_string "ISCONVEX ? " ;
do_list (fun i -> print_int i ; print_string " ") line_id_lst ;
print_string "\n" ;*)
 match line_id_lst with
    [] -> true
  | line_id::q ->
     if (List.exists (fun x -> x = (findLinedefNext line_id)) original_list)
      then
       begin
       let line1start = findLinedefStart line_id in
       let line1end = findLinedefEnd line_id in
       let line2start = findLinedefStart (findLinedefNext line_id) in
       let line2end = findLinedefEnd (findLinedefNext line_id) in
        match (getSegPosition line1start line1end line2start line2end) with
           Right -> isConvexPolygon original_list q
         | Left -> false
       end
      else isConvexPolygon original_list q
;;



(* ******************************************************** *)
(* RETOURNE LE NUMERO DU SECTEUR AUQUEL APPARTIENT LA LIGNE *)
(* ******************************************************** *)
let findLinedefSector n =
 let line = globallinedeftable.(n) in
  line.sector
;;


(* *************************************************************** *)
(* FONCTION QUI REGARDE SI TOUTES LES LIGNES DONT LES NUMEROS SONT *)
(* DANS LA LISTE linelst APPARTIENNENT AU MEME SECTEUR.            *)
(* int list -> bool                                                *)
(* *************************************************************** *)
let isSingleSectorLines line_id_lst =
 match line_id_lst with
    [] -> true
  | line_id::q ->
    let sector = findLinedefSector line_id in
     let rec recIsSingle lst =
      match lst with
         [] -> true
       | h::q -> if (findLinedefSector h) = sector
                  then recIsSingle q
                  else false
      in
       recIsSingle line_id_lst
;;


(* ******************************************************************* *)
(* FONCTION QUI PREND UNE LISTE DE NUMEROS DE LIGNES ET QUI RENDS DEUX *)
(* LISTES DE NUMEROS DE LIGNES : lleft ET lright.                      *)
(* lleft EST LA LISTE DES LIGNES SE TROUVANT A GAUCHE DE LA LIGNE DE   *)
(* PARTITION partline.                                                 *)
(* lright EST LA LISTE DES LIGNES SE TROUVANT A DROITE DE LA LIGNE DE  *)
(* PARTITION partline.                                                 *)
(* int -> int list -> (int list * int list)                            *)
(* ******************************************************************* *)
let partitionList partline_id line_id_lst =
 let partlinestart = findLinedefStart partline_id in
 let partlineend = findLinedefEnd partline_id in
 let rec recpartitionList lst acculeft accuright =
  match lst with
     [] -> (acculeft, accuright)
   | h::q ->
      let segstart = findLinedefStart h in
      let segend = findLinedefEnd h in
       match (getSegPosition partlinestart partlineend  segstart segend) with
          Right -> recpartitionList q acculeft (h::accuright)
        | Left -> recpartitionList q (h::acculeft) accuright
 in
  recpartitionList line_id_lst [] []
;;

