(* CHARGEMENT DE LA VERSION BINAIRE CAML DU MONDE COMPLET *)
(* CAD AVEC BSP. DONNEES SORTIES DU COMPILATEUR DE BSP ET *)
(* UTILISABLES DIRECTEMENT PAR LE RENDERER OCAML.         *)


open Vertexsbase ;;
open Linedefsbase ;;
open Sectorsbase ;;
open Bsptreebase ;;


let bLevelLoadVertexes in_channel =
 let header = (input_line in_channel) in
  if header <> "VERTEXES"
   then
    raise (Invalid_argument "Bad file type - No vertexes header")
   else
    begin
    let nbvexterxes = int_of_string (input_line in_channel) in
     for i=0 to nbvexterxes-1
      do
      addVertex (input_value in_channel) ;
      done
    end
;;

(* ******************************************************** *)
(* CHARGEMENT DES LINEDEFS EN MEMOIRE A PARTIR D'UN FICHIER *)
(* ******************************************************** *)
let bLevelLoadLinedefs in_channel =
 let header = (input_line in_channel) in
  if header <> "LINEDEFS"
   then
    raise (Invalid_argument "Bad file type - No linedef header")
   else
    begin
    let nbLinedefs = int_of_string (input_line in_channel) in
     for i=0 to nbLinedefs-1
      do
        addLinedef (input_value in_channel) ;
      done
    end
;;


(* ******************************************************** *)
(* CHARGEMENT DES SECTEURS EN MEMOIRE A PARTIR D'UN FICHIER *)
(* ******************************************************** *)
let bLevelLoadSectors in_channel =
 let header = (input_line in_channel) in
  if header <> "SECTORS"
   then
    raise (Invalid_argument "Bad file type - No sectors header")
   else
    begin
    let nbSectors = int_of_string (input_line in_channel) in
     for i=0 to nbSectors-1
      do
      addSector (input_value in_channel) ;
      done
    end
;;


let bLevelLoadBsp in_channel =
 let header = (input_line in_channel) in
  if header <> "BSP-PART"
   then
    raise (Invalid_argument "Bad file type - No bsp header")
   else
    input_value in_channel
;;


(* CHARGEMENT BINAIRE DE LA BLOCKMAP *)
let bLevelLoadBlockMap in_channel =
 let header = input_line in_channel in
  if header <> "BLOCKMAP"
   then raise (Invalid_argument "Bad file type - No blockmap header")
   else
    begin
    print_string "Blockmap width : " ;
    print_int (int_of_string (input_line in_channel)) ;
    print_string "\n" ;
    print_string "Blockmap height : " ;
    print_int (int_of_string (input_line in_channel)) ;
    print_string "\n" ;
    input_value in_channel
    end
;;



let bLevelLoadLevel filename =
 let in_channel = open_in filename in
  let header = input_line in_channel in
   if header <> "LEVEL-FILE-BINARY"
    then raise (Invalid_argument "Bad file type - No main header")
    else
     begin
     print_string "Loading external level...\n" ;
     Format.print_flush () ;
     bLevelLoadVertexes in_channel ;
     print_string "Vertices loaded\n" ;
     Format.print_flush () ;
     bLevelLoadLinedefs in_channel ;
     print_string "Linedefss loaded\n" ;
     Format.print_flush () ;
     bLevelLoadSectors in_channel ;
     print_string "Sectors loaded\n" ;
     Format.print_flush () ;
     let tree = bLevelLoadBsp in_channel in
     print_string "BSP loaded\n" ;
     Format.print_flush () ;
     let blockmap = bLevelLoadBlockMap in_channel in
     print_string "Blockmap loaded\n" ;
     Format.print_flush () ;
      close_in in_channel ;
      (tree, blockmap)
     end
;;

