open Floatextra ;;
open Bclinedefs ;;
open Linedefsbase ;;
open Bcbsptree ;;



(* PERMET DE SAVOIR S'IL Y A INTERSECTION ENTRE 2 SEGMENTS *)
(* L'INTERSECTION DOIT SE TROUVER <A L'INTERIEUR> DES 2 SEGMENTS *)
let existSegmentRealIntersect xa ya xb yb xc yc xd yd =
  let denom = (xb-.xa) *. (yd-.yc) -. (yb-.ya) *. (xd-.xc) in
   if float_eq denom 0.0
    then false
    else
     begin
     (* IL FAUT REGARDER SI L'INTERSECTION A BIEN LIEU *)
     (* ENTRE LES EXTREMITES DES 2 SEGMENTS. *)
     let s = ((ya-.yc) *. (xb-.xa) -. (xa-.xc) *. (yb-.ya)) /. denom in
      if (s <= 1.0 && s >= 0.0) then
       let r = ((ya-.yc) *. (xd-.xc) -. (xa-.xc) *. (yd-.yc)) /. denom in
        (r <= 1.0 && r >= 0.0)
      else false
     end
;;



(* ROUTINE DE VERIFICATION DE SI UNE LIGNE INTERSECTE UN RECTANGLE *)
let intersectsBox line_id xbs ybs xbe ybe =
 let (x1,y1) = findLinedefStart line_id
 and (x2,y2) = findLinedefEnd line_id in
  (* INTERSECTION AVEC L'UN DES COTES ? *)
  (existSegmentRealIntersect x1 y1 x2 y2 xbs ybs xbe ybs) ||
  (existSegmentRealIntersect x1 y1 x2 y2 xbe ybs xbe ybe) ||
  (existSegmentRealIntersect x1 y1 x2 y2 xbe ybe xbs ybe) ||
  (existSegmentRealIntersect x1 y1 x2 y2 xbs ybe xbs ybs)
;;


let isInBox line_id xbs ybs xbe ybe =
 let (x1,y1) = findLinedefStart line_id
 and (x2,y2) = findLinedefStart line_id in
  let xmin = min x1 x2
  and ymin = min y1 y2
  and xmax = max x1 x2
  and ymax = max y1 y2 in
   (xmin >= xbs && xmax <= xbe && ymin >= ybs && ymax <= ybe)
;;



(* CONSTRUCTION DE LA LISTE DES LINEDEFS PASSANT DANS 1 CASE *)
(* DE LA BLOCKMAP.                                           *)
let buildLinesList xblockstart yblockstart xblockend yblockend =
 (* ACCUMULATEUR D'IDENTIFICATEURS DE LINEDEFS *)
 let line_lst = ref [] in
  for i = 0 to !nbLinedefs - 1
   do
   (* ON NE PREND EN COMPTE QUE LES LINEDEFS IMPASSABLES *)
   if not ((globallinedeftable.(i)).passable) then
    if (intersectsBox i xblockstart yblockstart xblockend yblockend) ||
       (isInBox i xblockstart yblockstart xblockend yblockend)
     then line_lst := i :: !line_lst
   done ;
  !line_lst
;;




(* ROUTINE PRINCIPALE DE CREATION DE LA BLOCKMAP *)
(* xmax yma : COORD MAX INCLUSES DU MONDE        *)
let computeBlockMap xmax ymax =
 (* xmin ymin ARE ASSUMED TO BE 0 *)
 (* ASSUME BLOCK SIZE 128x128 *)
 (* NOMBRE DE BLOCKS NECESSAIRES *)
 let horiz_block = 1 + truncate (xmax /. 128.0)
 and vert_block = 1 + truncate (ymax /. 128.0) in
  (* TABLEAU VERTICAL DE "POINTEURS" : <LE> POINT D'ENTREE DS LA BLOCKMAP *)
  let vert_ptr = Array.create vert_block [||] in
   begin
   (* TABLEAUX HORIZONTAUX *)
   for i = 0 to vert_block - 1
    do vert_ptr.(i) <- Array.create horiz_block [-1] done ;

   (* INITIALISATION DES LISTES D'IDENTIFICATEURS DE LINEDEFS *)
   for i = 0 to vert_block - 1
    do
    for j = 0 to horiz_block - 1
     do
     (* EXTREMITES <INCLUSES> *)
     let xblockstart = 128.0 *. (float j)
     and yblockstart = 128.0 *. (float i) in
     let xblockend = xblockstart +. 127.0
     and yblockend = yblockstart +. 127.0 in
     (vert_ptr.(i)).(j) <-
            buildLinesList xblockstart yblockstart xblockend yblockend
     done
    done ;
   (* ON RETOURNE UN TRIPLET : (NB BLOCKS HORIZ, NB BLOCKS VERT, BLOCKMAP) *)
   (horiz_block, vert_block, vert_ptr)
   end
;;


(* GENERATION DE LA BLOCK MAP - POINT D'ENTREE - *)
let buildBlockMapFromBSP tree =
 match compute_bounding_box tree with
    Some (_,_,xmax,ymax) -> computeBlockMap xmax ymax
  | None -> raise (Invalid_argument "buildBlockMapFromBSP - Empty world")
;;

