open Tk ;;
open Linedefsbase ;;
open Vertexsbase ;;
open Bclinedefs ;;
open Editorextern ;;



let findOtherSide v_id1 v_id2 =
 (* ON DOIT RECHERCHER DANS LA LISTE DES LIGNES S'IL Y EN A UNE QUI A LES *)
 (* MEMES VERTEX, MAIS DANS LE SENS INVERSE.                              *)
 (* (MAINTX, UPPERTX, LOWERTX)                                            *)
 let rec parse index =
  if index > !nbLinedefs
   then None
   else
    begin
    let theline = globallinedeftable.(index) in
     if theline.src = v_id2 && theline.dest = v_id1
      then Some (index)
      else parse (index+1)
    end
 in parse 0
;;


let newLinedef widget scale sector_id v_id1 v_id2 =
 let v1 = globalvertextable.(v_id1)
 and v2 = globalvertextable.(v_id2) in
  let xstart = Pixels (truncate (v1.x /. scale))
  and ystart = Pixels (truncate (v1.y /. scale))
  and xend = Pixels (truncate (v2.x /. scale))
  and yend = Pixels (truncate (v2.y /. scale)) in
   let line_toi = Canvas.create_line widget [xstart;ystart;xend;yend]
                                               [ArrowStyle Arrow_Last] in
    (* ON CREE LA LINEDEF SANS ENCORE POUVOIR ENCORE *)
    (* LES CHAINER ET ON LA RAJOUTE DANS LA TABLE.   *)
    let l_id = addLinedef { src = v_id1 ; dest = v_id2 ;
                            sector = sector_id ; next = -1 ;
                            angle = getLineAngle v_id1 v_id2 ;
                            len = getLineLen  v_id1 v_id2 ;
                            adjacentsector = -1 ;
                            maintx = 0 ; uppertx = 255 ; lowertx = 255 ;
                            passable = false } in
     linedefWidgetTable.(l_id) <- line_toi ;
     (* ON RETOURNE LE NUMERO DE LA LINEDEF *)
     l_id
;;



(* RECHERCHE LA DISTANCE D'UN POINT A UNE DROTE *)
let findPointToLineDist xc yc line_id =
 let line = globallinedeftable.(line_id) in
  let (xa, ya) = findLinedefStart line_id
  and (xb, yb) = findLinedefEnd line_id
  and linelen = line.len in
   let r = ((ya-.yc)*.(ya-.yb)-.(xa-.xc)*.(xb-.xa)) /. (linelen*.linelen) in
    let xi = xa+.r*.(xb-.xa)
    and yi = ya+.r*.(yb-.ya) in
     sqrt((xc-.xi)*.(xc-.xi)+.(yc-.yi)*.(yc-.yi))
;;


let findNearestLine pointx pointy =
 let rec findRec index nearestdist nearestlineid =
  if (index >= !nbLinedefs) then nearestlineid
  else
   let dist = findPointToLineDist pointx pointy index in
    if dist < nearestdist then
     findRec (index+1) dist index
    else
     findRec (index+1) nearestdist nearestlineid
 in
  (* C'EST PAS GENIAL COMME INIT CAR SI ON N'A AUCUNE LIGNE INFERIEURE *)
  (* EN DISTANCE A 100000.0, CA PLANTERA. BON C'EST PEU PROBABLE, MAIS *)
  (* QUAND MEME... POUR LES PREUVES DE SURETE ON REPASSERA...          *)
  match findRec 0 100000.0 (-1) with
     -1 -> raise Not_found
   | other -> other
;;


let rec updateDoubleSidedLinedefs = function
   [] -> ()
 | h_id::q ->
    begin
    let line = globallinedeftable.(h_id) in
     match findOtherSide line.src line.dest with
        None -> updateDoubleSidedLinedefs q
      | Some l_id ->
         (globallinedeftable.(h_id)).adjacentsector <-
                                    ((globallinedeftable.(l_id)).sector) ;
         (globallinedeftable.(l_id)).adjacentsector <-
                                   ((globallinedeftable.(h_id)).sector)
    end
;;


let openSingleSideWindow parentwidget marker_rect line_id =
 let line = globallinedeftable.(line_id)
 and requester_window = Toplevel.create parentwidget [] in
  Wm.title_set requester_window ("Single sided linedef") ;
  Focus.set requester_window ;
  let frame0 = Frame.create requester_window [Relief Raised]
  and frame1 = Frame.create requester_window [Relief Raised] in
   let lineidLabel = Label.create frame0 [Relief Groove;
                               Text ("Linedef #"^(string_of_int line_id))]
   and mTxLabel = Label.create frame0 [Text "Main Texture";Relief Groove]
   and mTxEntry = Entry.create frame0 [Relief Sunken]
   and adjSect1_0Label = Label.create frame0 [Text "Adjacent Sector";
                                                Relief Groove]
   and adjSect1_1Label = Label.create frame0 [Relief Groove;
       Text (match line.adjacentsector with -1 -> "Disabled"
                                           | i -> string_of_int i)] in
   let passableVariable = Textvariable.create_temporary frame0 in
   let passableLabel = Label.create frame0 [Text "Permeability";Relief Groove]
   and passableChkButt = Checkbutton.create frame0 [Text "Passable" ;
                                                  Variable passableVariable]
   and okButton = Button.create frame1 [Text "Ok"]
   and cancelButton = Button.create frame1 [Text "Cancel";
                          Command (fun () ->
                                    Canvas.delete parentwidget [marker_rect] ;
                                    destroy requester_window)]
  in
   pack [frame0;frame1] [Fill Fill_X] ;
   pack [lineidLabel;mTxLabel;mTxEntry;adjSect1_0Label;adjSect1_1Label;
         passableLabel; passableChkButt] [Fill Fill_X] ;
   pack [okButton;cancelButton] [Fill Fill_X] ;
   (* INIT DES CHAMPS DU REQUESTERS *)
   Entry.insert mTxEntry End (string_of_int line.maintx) ;
   if line.passable then Checkbutton.toggle passableChkButt ;
   (* ---------- CONTINUATIONS ------------ *)
   let continueFct () =
    (globallinedeftable.(line_id)).maintx <-
                                  int_of_string (Entry.get mTxEntry) ;
    (globallinedeftable.(line_id)).passable <- Textvariable.get passableVariable = "1" ;
    Canvas.delete parentwidget [marker_rect] ;
    destroy requester_window
   in
    Button.configure okButton [Command continueFct]
;;




let openDoubleSideWindow parentwidget marker_rect line1_id line2_id =
 let line1 = globallinedeftable.(line1_id)
 and line2 = globallinedeftable.(line2_id)
 and requester_window = Toplevel.create parentwidget [] in
  Wm.title_set requester_window "Double sided linedef" ;
  Focus.set requester_window ;
  let frame0 = Frame.create requester_window [Relief Raised]
  and frame1 = Frame.create requester_window [Relief Raised] in
  let frame2 = Frame.create frame0 [Relief Raised]
  and frame3 = Frame.create frame0 [Relief Raised]
   in
   let lineid1Label = Label.create frame2 [Relief Groove;
                               Text ("Linedef #"^(string_of_int line1_id))]
   and uTx1Label = Label.create frame2 [Text "Upper Texture";Relief Groove]
   and uTx1Entry = Entry.create frame2 [Relief Sunken]
   and mTx1Label = Label.create frame2 [Text "Main Texture";Relief Groove]
   and mTx1Entry = Entry.create frame2 [Relief Sunken]
   and lTx1Label = Label.create frame2 [Text "Lower Texture";Relief Groove]
   and lTx1Entry = Entry.create frame2 [Relief Sunken]
   and adjSect1_0Label = Label.create frame2 [Text "Adjacent Sector";
                                                Relief Groove]
   and adjSect1_1Label = Label.create frame2 [Relief Groove;
       Text (match line1.adjacentsector with -1 -> "Disabled"
                                           | i -> string_of_int i)] in
   let passableVariable1 = Textvariable.create_temporary frame2 in
   let passableLabel1 = Label.create frame2 [Text "Permeability";Relief Groove]
   and passableChkButt1 = Checkbutton.create frame2 [Text "Passable" ;
                                                  Variable passableVariable1]
   (* SECONDE "SIDE" DE LA LINEDEF *)
   and lineid2Label = Label.create frame3 [Relief Groove;
                               Text ("Linedef #"^(string_of_int line2_id))]
   and uTx2Label = Label.create frame3 [Text "Upper Texture";Relief Groove]
   and uTx2Entry = Entry.create frame3 [Relief Sunken]
   and mTx2Label = Label.create frame3 [Text "Main Texture";Relief Groove]
   and mTx2Entry = Entry.create frame3 [Relief Sunken]
   and lTx2Label = Label.create frame3 [Text "Lower Texture";Relief Groove]
   and lTx2Entry = Entry.create frame3 [Relief Sunken]
   and adjSect2_0Label = Label.create frame3 [Text "Adjacent Sector";
                                                Relief Groove]
   and adjSect2_1Label = Label.create frame3 [Relief Groove;
       Text (match line2.adjacentsector with -1 -> "Disabled"
                                            | i -> string_of_int i)] in
   let passableVariable2 = Textvariable.create_temporary frame3 in
   let passableLabel2 = Label.create frame3 [Text "Permeability";Relief Groove]
   and passableChkButt2 = Checkbutton.create frame3 [Text "Passable" ;
                                                  Variable passableVariable2]
   (* BOUTONS DE SORTIE *)
   and okButton = Button.create frame1 [Text "Ok"]
   and cancelButton = Button.create frame1 [Text "Cancel";
                          Command (fun () ->
                                     Canvas.delete parentwidget [marker_rect] ;
                                     destroy requester_window)]
    in
    pack [frame0;frame1] [Fill Fill_X] ;
    pack [frame2;frame3] [Side Side_Left;Fill Fill_X] ;
    pack [lineid1Label;uTx1Label;uTx1Entry;mTx1Label;mTx1Entry;
          lTx1Label;lTx1Entry;adjSect1_0Label;adjSect1_1Label;
          passableLabel1;passableChkButt1] [Fill Fill_X] ;
    pack [lineid2Label;uTx2Label;uTx2Entry;mTx2Label;mTx2Entry;
          lTx2Label;lTx2Entry;adjSect2_0Label;adjSect2_1Label;
          passableLabel2;passableChkButt2] [Fill Fill_X] ;
    pack [okButton;cancelButton] [Fill Fill_X] ;
    (* INIT DES CHAMPS DU REQUESTERS *)
    Entry.insert uTx1Entry End (string_of_int line1.uppertx) ;
    Entry.insert mTx1Entry End (string_of_int line1.maintx) ;
    Entry.insert lTx1Entry End (string_of_int line1.lowertx) ;
    if line1.passable then Checkbutton.toggle passableChkButt1 ;
    Entry.insert uTx2Entry End (string_of_int line2.uppertx) ;
    Entry.insert mTx2Entry End (string_of_int line2.maintx) ;
    Entry.insert lTx2Entry End (string_of_int line2.lowertx) ;
    if line2.passable then Checkbutton.toggle passableChkButt2 ;
    (* ---------- CONTINUATIONS ------------ *)
    let continueFct () =
     begin
     (* PREMIERE LINEDEF *)
     (globallinedeftable.(line1_id)).uppertx <-
                                  int_of_string (Entry.get uTx1Entry) ;
     (globallinedeftable.(line1_id)).maintx <-
                                  int_of_string (Entry.get mTx1Entry) ;
     (globallinedeftable.(line1_id)).lowertx <-
                                  int_of_string (Entry.get lTx1Entry) ;
     (globallinedeftable.(line1_id)).passable <-
                                  Textvariable.get passableVariable1 = "1" ;
     (* SECONDE LINEDEF *)
     (globallinedeftable.(line2_id)).uppertx <-
                                  int_of_string (Entry.get uTx2Entry) ;
     (globallinedeftable.(line2_id)).maintx <-
                                  int_of_string (Entry.get mTx2Entry) ;
     (globallinedeftable.(line2_id)).lowertx <-
                                  int_of_string (Entry.get lTx2Entry) ;
     (globallinedeftable.(line2_id)).passable <-
                                  Textvariable.get passableVariable2 = "1" ;
     Canvas.delete parentwidget [marker_rect] ;
     destroy requester_window
     end
    in
     Button.configure okButton [Command continueFct]
;;



let editLinedef widget scale pointx pointy =
 try
 let nearestline_id = findNearestLine (pointx*.scale) (pointy*.scale) in
  let nearestline = globallinedeftable.(nearestline_id)
  and (xa, ya) = findLinedefStart nearestline_id
  and (xb, yb) = findLinedefEnd nearestline_id in
   let rect_toi = Canvas.create_rectangle widget
              (Pixels (truncate (xa/.scale)))
              (Pixels (truncate (ya/.scale)))
              (Pixels (truncate (xb/.scale)))
              (Pixels (truncate (yb/.scale))) [Outline Red]
    in
    (* ON RECHERCHE SI CETTE LIGNE A UNE SECONDE FACE *)
    (* MATERIALISEE PAR UNE AUTRE LIGNE AVEC DES VERTICES OPPOSES *)
    (* EN TERME DE START ET DE STOP.                              *)
    match findOtherSide nearestline.src nearestline.dest with
       None -> openSingleSideWindow widget rect_toi nearestline_id
     | Some (otherline_id) ->
         openDoubleSideWindow widget rect_toi nearestline_id otherline_id
 with Not_found -> ()
;;


let handleLinedefSelection widget widgetinfo scale pointx pointy =
 try
 let nearestline_id = findNearestLine (pointx*.scale) (pointy*.scale) in
  let nearestline = globallinedeftable.(nearestline_id) in
   (* ON VIRE L'ANCIENNE INFO *)
   Text.delete widgetinfo (TextIndex ((LineChar (0,0)), []))
                                                 (TextIndex (End, [])) ;
   (* ON MET LA NOUVELLE *)
   Text.insert widgetinfo (TextIndex (End, []))
                 ("Linedef #"^(string_of_int nearestline_id)^"\n") [] ;
   Text.insert widgetinfo (TextIndex (End, []))
                 ("Start vertex #"^(string_of_int nearestline.src)^"\n") [] ;
   Text.insert widgetinfo (TextIndex (End, []))
                 ("End vertex #"^(string_of_int nearestline.dest)^"\n") [] ;
   Text.insert widgetinfo (TextIndex (End, []))
                 ("Upper Tx #"^(if nearestline.uppertx = 255 then "NONE"
                                else string_of_int nearestline.uppertx)^
                                "\n") [] ;
   Text.insert widgetinfo (TextIndex (End, []))
                 ("Main Tx #"^(if nearestline.maintx = 255 then "NONE"
                                else string_of_int nearestline.uppertx)^
                                "\n") [] ;
   Text.insert widgetinfo (TextIndex (End, []))
                 ("Lower Tx #"^(if nearestline.lowertx = 255 then "NONE"
                                else string_of_int nearestline.uppertx)^
                                "\n") [] ;
   Text.insert widgetinfo (TextIndex (End, []))
                 ("Passable :"^(string_of_bool nearestline.passable)) []
 with Not_found -> ()
;;

