open Tk ;;
open Linedefsbase ;;
open Bclinedefs ;;
open Editline ;;
open Requester ;;
open Sectorsbase ;;
open Editvertex ;;
open Editorextern ;;
open Location ;;


(* ***************************************** *)
(* CHAINE LES LIGNES D'UN SECTEUR APRES COUP *)
(* ***************************************** *)
let chainLines l =
 let first_line_id = List.hd l in
  let rec recChainLines = function
     [] -> first_line_id
   | l_id::q ->
      let nextl_id = recChainLines q in
       let old_line = globallinedeftable.(l_id) in
        (globallinedeftable.(l_id)).next <- nextl_id ;
       l_id
  in recChainLines l
;;


(* FONCTION QUI REND LE NUMERO QUI SERA ATTRIBUE A LA PROCHAINE *)
(* CREATION DE SECTEUR. C'EST UN PEU UNE VUE DE L'AVENIR...     *)
let getNextSectorNumber () =
 !nbSectors
;;



(* ******************************************************* *)
(* FABRIQUE UN SECTEUR A PARTIR DE LA SUITE DE VERTEXES    *)
(* QUI FORMENT LES POINTS D'ATTACHE DES LINEDEFS CONTOURS. *)
(* RETOURNE LA LISTE DES LINEDEFS FORMANT LE SECTEUR. CES  *)
(* LINEDEFS SONT CREES A PARTIR DE LE LISTE DES VERTICES   *)
(* PASSEE EN PARAMETRE.                                    *)
(* ******************************************************* *)
let rec createSectorFromVertList widget sect_id = function
   [] ->
    raise (Invalid_argument "createSectorFromVertList")
 | h::[] ->
    (* IL FAUT AU MOINS 2 POINTS ET ILS SONT TOUJOURS PRIS 2 PAR 2. *)
    (* LE POINT D'ARRIVE (= A CELUI DE DEPART) N'A PAS ETE FOURNI.  *)
    raise (Invalid_argument "createSectorFromVertList")
 | h::q::rest ->
    let l_id = newLinedef widget !scaleReducing sect_id h q in
     if rest <> []
      then l_id :: (createSectorFromVertList widget sect_id (q::rest))
      else [l_id]
;;


(* LANCE LA CREATION DU SECTEUR *)
let createTheNewSector widget pointlst =
 match pointlst with
    [] -> raise (Invalid_argument "createTheNewSector")
  | h::[] -> raise (Invalid_argument "createTheNewSector")
  | goodlist ->
     (* ----------- CONTINUATIONS ----------- *)
     let continueFct s1 s2 =
      (* ON RECUPERE LA LISTE DES LINEDEFS COMPOSANTE LE SECTEUR *)
      (* CES LINEDEFS SONT CREES PAR EFFET DE BORD *)
      let lines_id_lst = createSectorFromVertList widget
                                          (getNextSectorNumber ()) goodlist in
       (* ON CHAINE LES LINEDEFS EN TERME DE SUCCESSEUR APRES COUP *)
       chainLines lines_id_lst ;
       (* ON MET A JOUR LES DOUBLE SIDED LINEDEF *)
       updateDoubleSidedLinedefs lines_id_lst ;
       (* MAINTENANT, ON PEUT RAJOUTER LE SECTEUR DANS LA TABLE DES SECTEURS *)
       addSector { bounds = lines_id_lst ; ceilingh = (float_of_string s1) ;
                   floorh = (float_of_string s2) } ; ()

     and cancelFct s1 s2 = infoRequester widget "Sector canceled"

     in
      twoStringRequester widget "Ceiling altitude" "Floor altitude"
                           continueFct cancelFct
;;


(* ACCUMULE LES VERTICES A REGROUPER EN SECTEUR *)
(* NE PREND PAS EN PARAMETRE L'ECHELLE CAR CELLE-CI *)
(* PEUT CHANGER EN COURS DE ROUTE. CA PERMET AINSI *)
(* DE POUVOIR CHANGER L'ECHELLE EN MODE DE CREATION *)
(* DE SECTEUR...                                    *)
let makeSector =
 let verticesLst = ref ([]:int list) in
 function widget -> function x -> function y ->
  let v_id = findNearestVertex (x*. !scaleReducing) (y*. !scaleReducing) in
   if (List.mem v_id !verticesLst) then
    begin
    (* ON CLOS LE SECTEUR *)
    verticesLst := !verticesLst @ [v_id] ;
    createTheNewSector widget !verticesLst ;
    (* ON REMET L'EPAISSEUR DES BORDS DES VERTICES COMME DE NORMAL *)
    List.iter (fun x -> Canvas.configure_rectangle widget vertexWidgetTable.(x)
                                     [Width (Pixels 1)])
               !verticesLst ;
    (* ON VIDE LA LISTE DES VERTICES A ASSEMBLER EN SECTEUR *)
    verticesLst := []
    end
   else
    begin
    (* ON CONTINUE LE SECTEUR EN RAJOUTANT LE VERTEX EN QUEUE *)
    verticesLst := !verticesLst @ [v_id] ;
    (* ON EPAISSIT LE BORD DU VERTEX POUR NOTIFIER LA PRISE EN COMPTE *)
    Canvas.configure_rectangle widget vertexWidgetTable.(v_id)
                [Width (Pixels 3)]
    end
;;



(* DIT SI UN POINT DE TROUVE A DROITE OU SUR UNE LINEDEF *)
let testRightNess pointx pointy l_id =
 let (xa,ya) = findLinedefStart l_id in
 let (xb,yb) = findLinedefEnd l_id in
  match getPointPositionAccurate (pointx,pointy) (xa,ya) (xb,yb) with
    P_On -> true
  | P_Right -> true
  | P_Left -> false
;;


(* RECHERCHE LE PREMIER SECTEUR POUR LEQUEL LE POINT PASSE *)
(* EN PARAMETRE SE TROUVE A L'INTERIEUR...                 *)
(* SI ON NE TROUVE RIEN, ON LEVE Not_found.                *)
let findNearestSector x y =
 let rec findRec index =
  if index >= !nbSectors then raise Not_found
  else
   begin
   (* ON VA REGARDER SI LE POINT EST A DROITE DE TOUTES LES *)
   (* LINEDEFS FORMANT LE SECTEUR. SI OUI, C'EST GAGNE.     *)
   if List.for_all (testRightNess x y) (globalsectortable.(index)).bounds
    then index
    else findRec (index+1)
   end
 in
   findRec 0
;;



let openSectorWindow widget s_id =
 let sect = globalsectortable.(s_id)
 and requester_window = Toplevel.create widget [] in
  Wm.title_set requester_window "Sector Properties" ;
  Focus.set requester_window ;
  let frame0 = Frame.create requester_window [Relief Raised] in
  let sectLabel = Label.create frame0 [Relief Groove;
                             Text ("Sector #"^(string_of_int s_id))]
  and ceilLabel = Label.create frame0 [Text "Ceiling Altitude";Relief Groove]
  and ceilEntry = Entry.create frame0 [Relief Sunken]
  and floorLabel = Label.create frame0 [Text "Floor Altitude";Relief Groove]
  and floorEntry = Entry.create frame0 [Relief Sunken]

  and okButton = Button.create frame0 [Text "Ok"]
  and cancelButton = Button.create frame0 [Text "Cancel";
                          Command (fun () -> destroy requester_window)]
  in
   pack [frame0][] ;
   pack [sectLabel;ceilLabel;ceilEntry;floorLabel;floorEntry;okButton;
         cancelButton] [Fill Fill_X] ;
   (* INIT DES CHAMPS DU REQUESTERS *)
   Entry.insert ceilEntry End (string_of_float sect.ceilingh) ;
   Entry.insert floorEntry End (string_of_float sect.floorh) ;
   (* ---------- CONTINUATIONS ------------ *)
   let continueFct () =
    begin
    (globalsectortable.(s_id)).ceilingh <-
                 (float_of_string (Entry.get ceilEntry)) ;
    (globalsectortable.(s_id)).floorh <-
                 (float_of_string (Entry.get floorEntry)) ;
    destroy requester_window
    end
   in Button.configure okButton [Command continueFct]
;;



let editSector widget scale x y =
 try
  openSectorWindow widget (findNearestSector (x*.scale) (y*.scale))
 with Not_found -> ()
;;


let handleSectorSelection widget widgetinfo scale x y =
 try
 let sect_id = (findNearestSector (x*.scale) (y*.scale)) in
 let the_sect = globalsectortable.(sect_id) in
  (* ON VIRE L'ANCIENNE INFO *)
  Text.delete widgetinfo (TextIndex ((LineChar (0,0)), []))
                                                 (TextIndex (End, [])) ;
  (* ON RAJOUTE LE NOUVELLE *)
  Text.insert widgetinfo (TextIndex (End, []))
                 ("Sector #"^(string_of_int sect_id)^"\n") [] ;
  Text.insert widgetinfo (TextIndex (End, []))
                 ("Ceiling = "^(string_of_float the_sect.ceilingh)^"\n") [] ;
  Text.insert widgetinfo (TextIndex (End, []))
                 ("Floor = "^(string_of_float the_sect.floorh)^"\n") []
  with Not_found -> ()
;;
