open Floatextra ;;



(* ********************************************************** *)
(* LATERALISATION D'UN POINT PAR RAPPORT A UNE LIGNE ORIENTEE *)
(* ********************************************************** *)
type t_Pointlocation =
   P_Right
 | P_Left
 | P_On
;;


(* ******************************************************************* *)
(* !!!!!!!!!! VERSION SE FOUTTANT DES ERREURS D'ARRONDI !!!!!!!!!!!!!! *)
(* !!!!!!!!!!!!!!!!!! SERT UNIQUEMENT QU RENDERER !!!!!!!!!!!!!!!!!!!! *)
(* FONCTION PERMETTANT DE SAVOIR SI UN POINT (px,py) EST A DROITE D'UN *)
(* VECTEUR.                                                            *)
(* ATTENTION SI LE POINT EST SUR LA LIGNE, IL EST CONSIDERE AUSSI A    *)
(* DROITE DE CELLE-CI.                                                 *)
(* px,py : ABSCISSE, ORDONNEE DU POINT                                 *)
(* x1,y1 : ABSCISSE, ORDONNEE DU POINT DE DEPART DU VECTEUR            *)
(* x2,y2 : ABSCISSE, ORDONNEE DU POINT D'ARRIVEE DU VECTEUR            *)
(* float * float -> float * float -> float * float -> T_Pointlocation  *)
(* ******************************************************************* *)
let getPointPosition (px, py) (x1, y1) (x2, y2) =
 if ((x1-.x2) *. (py-.y2)) = ((y1-.y2) *. (px-.x2))
  then P_On
  else
   begin
   if ((x1-.x2) *. (py-.y2)) > ((y1-.y2) *. (px-.x2))
    then P_Right
    else P_Left
   end
;;


let getPointPositionAccurate (px, py) (x1, y1) (x2, y2) =
 if float_eq ((x1-.x2) *. (py-.y2)) ((y1-.y2) *. (px-.x2))
  then P_On
  else
   begin
   if ((x1-.x2) *. (py-.y2)) > ((y1-.y2) *. (px-.x2))
    then P_Right
    else P_Left
   end
;;



(* *********************************************************** *)
(* LATERALISATION D'UNE LIGNE PAR RAPPORT A UNE LIGNE ORIENTEE *)
(* *********************************************************** *)
type t_Linelocation =
   Left        (* GAUCHE *)
 | Right       (* DROITE *)
;;


(* ********************************************************************* *)
(* FONCTION PERMETTANT DE SAVOIR SI UN SEGMENT EST A DROITE D'UN VECTEUR *)
(* ssx,ssy : ABSCISSE, ORDONNEE DU POINT DE DEPART DU SEGMENT            *)
(* sex,sey : ABSCISSE, ORDONNEE DU POINT D'ARRIVEE DU SEGMENT            *)
(* lsx,lsy : ABSCISSE, ORDONNEE DU POINT DE DEPART DE LA DROITE          *)
(* lex,ley : ABSCISSE, ORDONNEE DU POINT D'ARRIVEE DE LA DROITE          *)
(* RAISE Invalid_argument "Intersection" SI UNE PARTIE EST A DROITE ET   *)
(* L'AUTRE A GAUCHE, CAD S'IL Y A INTERSECTION.                          *)
(* float * float -> float * float -> float * float ->                    *)
(*                                          float * float -> T_Location  *)
(* ********************************************************************* *)
let getSegPosition (lsx,lsy) (lex,ley) (ssx,ssy) (sex,sey) =
 match ((getPointPositionAccurate (ssx,ssy) (lsx,lsy) (lex,ley)),
        (getPointPositionAccurate (sex,sey) (lsx,lsy) (lex,ley))) with
    (P_Left, P_Left) -> Left
  | (P_Left, P_Right) -> raise (Invalid_argument "Intersection LR")
  | (P_Left, P_On) -> Left
  | (P_Right, P_Left) -> raise (Invalid_argument "Intersection RL")
  | (P_Right, P_Right) -> Right
  | (P_Right, P_On) -> Right
  | (P_On, P_Right) -> Right
  | (P_On, P_Left) -> Left
  | (P_On, P_On) ->
    (* ON VA FAIRE LE PRODUIT SCALAIRE DES 2 VECTEURS. S'IL EST *)
    (* POSITIF ALORS LES 2 VECTEURS VARIENT DANS LE MEME SENS ET *)
    (* ALORS LE SEGMENT EST A DROITE DE LA LIGNE DE PARTITION, *)
    (* SINON C'EST QUE LES VECTEURS SONT DE SENS OPPOSES ET LE *)
    (* SEGMENT EST A GAUCHE DE LA LIGNE DE PARTITION. *)
    (* ON ASSUME QUE LES LIGNES SONT ORIENTEES DANS LE SENS DES *)
    (* AIGUILLES D'UNE MONTRE. *)
    let scalprod = (lex-.lsx)*.(sex-.ssx) +. (ley-.lsy)*.(sey-.ssy) in
     if scalprod > 0.0 then Right else Left
;;
