open Tk ;;
open Editorextern ;;
open Ascsavedescr ;;
open Ascloaddescr ;;
open Requester ;;
open Vertexsbase ;;
open Linedefsbase ;;
open Sectorsbase ;;
open Editvertex ;;
open Editsector ;;
open Editline ;;
open Redraw ;;



(* SURFACE DE DESSIN *)
let mapTop = openTk () ;;
Wm.title_set mapTop "Editlevel" ;;
let mapCanvas = Canvas.create mapTop [Width (Pixels 800);
  Height (Pixels 600); BorderWidth (Pixels 3); Relief Raised;
  Background (NamedColor "seashell")] ;;
(* BARRE D'OUTILS *)
let frame1 = Frame.create mapTop [BorderWidth (Pixels 3); Relief Groove] ;;
let modeLabel = Label.create frame1 [Text "Edit modes"; Relief Groove] ;;
let lineRButton = Radiobutton.create frame1 [Text "Linedef mode";
  Value "L"; Command (fun () -> editState := EST_LINEDEF)] ;;
let sectorRButton = Radiobutton.create frame1 [Text "Sector mode";
  Value "S"; Command (fun () -> editState := EST_SECTOR)] ;;
let vertexRButton = Radiobutton.create frame1 [Text "Vertex mode";
  Value "V"; Command (fun () -> editState := EST_VERTEX)] ;;
Radiobutton.select vertexRButton ;;
let miscLabel = Label.create frame1 [Text "Miscellaneous"; Relief Groove] ;;
let loadButton = Button.create frame1 [Text "Load"] ;;
let saveButton = Button.create frame1 [Text "Save"] ;;
let quitButton = Button.create frame1 [Text "Quit"] ;;
let frame2 = Frame.create mapTop [] ;;
let infoLabel = Label.create frame1 [Text "Infos"; Relief Groove] ;;
let infoText = Text.create frame1 [Relief Raised;TextWidth 20;TextHeight 19] ;;
let scaleLabel = Label.create frame1 [Text "Scale"] ;;
let scaleScale = Scale.create frame1 [From 1.0; To 10.0; Orient Horizontal ;
                                      ShowValue true] ;;
pack [mapCanvas] [Side Side_Left] ;;
pack [frame1; modeLabel; lineRButton; sectorRButton; vertexRButton;
      miscLabel;loadButton;saveButton; quitButton]
     [Side Side_Top;Fill Fill_X] ;;
pack [frame2;infoLabel;infoText] [Fill Fill_Both] ;;
pack [scaleLabel;scaleScale] [Fill Fill_Both] ;;


(* BINDING DU SCALER *)
let rescaleMap s =
 scaleReducing := s ;
 Redraw.redrawMap mapCanvas s
;;
Scale.configure scaleScale [ScaleCommand rescaleMap] ;;


(* BINDING DU BOUTON DE SAUVEGARDE *)
let saveButtonBinding () =
 let continueFct filename = 
  aDescrSaveDescription (filename^".world") ;
  infoRequester mapTop ("Written : "^(string_of_int !nbVertexes)^
   " vertexes, "^(string_of_int !nbLinedefs)^" linedefs, "^
   (string_of_int !nbSectors)^" sectors.")
 and cancelFct filename = ()
 in
  singleStringRequester mapTop "Filename : " continueFct cancelFct
;;
Button.configure saveButton [Command saveButtonBinding] ;;



(* BINDING DU BOUTON DE CHARGEMENT *)
let loadButtonBinding () =
 let continueFct filename = 
  aDescrLoadDescription filename ;
  refreshAfterLoad mapCanvas !scaleReducing ;
  infoRequester mapTop ("Read : "^(string_of_int !nbVertexes)^
   " vertexes, "^(string_of_int !nbLinedefs)^" linedefs, "^
   (string_of_int !nbSectors)^" sectors.")
 and cancelFct filename = ()
 in
  singleStringRequester mapTop "Filename : " continueFct cancelFct
;;
Button.configure loadButton [Command loadButtonBinding] ;;


(* BINDING DU BOUTON DE SORTIE *)
let quitButtonBinding () =
 let continueFct () = exit (0)
 and cancelFct () = ()
in
 yesNoRequester mapTop "Really want to quit ?" continueFct cancelFct
;;
Button.configure quitButton [Command quitButtonBinding] ;;


(* BINDING DE LA TOUCHE shift ENFONCEE *)
let shiftKeyDownBinding xev =
 isShiftKeyPushed := true
;;
Tk.bind mapCanvas [([], KeyPressDetail "Shift_L")]
 (BindSet ([], shiftKeyDownBinding)) ;;


(* BINDING DE LA TOUCHE shift RELEVEE *)
let shiftKeyUpBinding xev =
 isShiftKeyPushed := false
;;
Tk.bind mapCanvas [([], KeyReleaseDetail "Shift_L")]
 (BindSet ([], shiftKeyUpBinding)) ;;


(* CLIC GAUCHE = SELECTION D'UN OBJET *)
let clic1Binding xev =
 match (!editState) with
    EST_VERTEX -> handleVertexSelection mapCanvas infoText !scaleReducing
           (Canvas.canvasx mapCanvas (Pixels xev.ev_MouseX))
           (Canvas.canvasy mapCanvas (Pixels xev.ev_MouseY))
  | EST_LINEDEF -> handleLinedefSelection mapCanvas infoText !scaleReducing
           (Canvas.canvasx mapCanvas (Pixels xev.ev_MouseX))
           (Canvas.canvasy mapCanvas (Pixels xev.ev_MouseY))
  | EST_SECTOR -> handleSectorSelection mapCanvas infoText !scaleReducing
           (Canvas.canvasx mapCanvas (Pixels xev.ev_MouseX))
           (Canvas.canvasy mapCanvas (Pixels xev.ev_MouseY))
;;
Tk.bind mapCanvas [([], ButtonPressDetail 1)]
 (BindSet ([Ev_MouseX;Ev_MouseY], clic1Binding)) ;;


(* CLIC DROIT = INSERTION OBJET *)
let clic3Binding xev =
 match (!editState) with
    EST_VERTEX ->
     createNewVertex mapCanvas !scaleReducing
      (truncate (Canvas.canvasx mapCanvas (Pixels xev.ev_MouseX)))
      (truncate (Canvas.canvasy mapCanvas (Pixels xev.ev_MouseY)))
  | EST_LINEDEF -> ()
  | EST_SECTOR -> makeSector mapCanvas (* NO !scaleReducing *)
           (Canvas.canvasx mapCanvas (Pixels xev.ev_MouseX))
           (Canvas.canvasy mapCanvas (Pixels xev.ev_MouseY))

;;
Tk.bind mapCanvas [([], ButtonPressDetail 3)]
 (BindSet ([Ev_MouseX;Ev_MouseY], clic3Binding)) ;;


(* DOUBLE CLIC = EDITION D'UN OBJET *)
let doubleClic1Binding xev =
 match (!editState) with
    EST_VERTEX -> raise (Invalid_argument "Not implemented")
  | EST_LINEDEF -> editLinedef mapCanvas !scaleReducing
          (Canvas.canvasx mapCanvas (Pixels xev.ev_MouseX))
          (Canvas.canvasy mapCanvas (Pixels xev.ev_MouseY))
  | EST_SECTOR -> editSector mapCanvas !scaleReducing
          (Canvas.canvasx mapCanvas (Pixels xev.ev_MouseX))
          (Canvas.canvasy mapCanvas (Pixels xev.ev_MouseY))
;;
Tk.bind mapCanvas [([Double], ButtonPressDetail 1)]
 (BindSet ([Ev_MouseX;Ev_MouseY], doubleClic1Binding)) ;;


(* GRILLE TEMPORAIRE *)
(*
Canvas.create_line mapCanvas [Pixels 127;Pixels 0;Pixels 127;Pixels 600] [] ;;
Canvas.create_line mapCanvas [Pixels 255;Pixels 0;Pixels 255;Pixels 600] [] ;;
Canvas.create_line mapCanvas [Pixels 383;Pixels 0;Pixels 383;Pixels 600] [] ;;
Canvas.create_line mapCanvas [Pixels 511;Pixels 0;Pixels 511;Pixels 600] [] ;;

Canvas.create_line mapCanvas [Pixels 0;Pixels 127;Pixels 800;Pixels 127] [] ;;
Canvas.create_line mapCanvas [Pixels 0;Pixels 255;Pixels 800;Pixels 255] [] ;;
Canvas.create_line mapCanvas [Pixels 0;Pixels 383;Pixels 800;Pixels 383] [] ;;
Canvas.create_line mapCanvas [Pixels 0;Pixels 511;Pixels 800;Pixels 511] [] ;;
*)

Focus.set mapCanvas ;;
mainLoop () ;;


 
