open Vertexsbase ;;
open Linedefsbase ;;
open Sectorsbase ;;
open Bsptreebase ;;


let printVertex v_id =
 let vert = globalvertextable.(v_id) in
  print_string "Vertex #" ; print_int v_id ;
  print_string " : (" ; print_float vert.x ;
  print_string " , " ; print_float vert.y ;
  print_string ")\n" ;
  Format.print_flush ()
;;


let printLinedef l_id =
 let line = globallinedeftable.(l_id) in
  print_string "Linedef #" ; print_int l_id ;
  print_string " : from " ; print_int line.src ;
  print_string " to " ; print_int line.dest ;
  print_string " in sect #" ; print_int line.sector ;
  print_string " next is #" ; print_int line.next ;
  print_string "\n" ;
  print_string "Angle is " ;
  print_float line.angle ;
  print_string "degrees\n" ;
  print_string "Len is " ;
  print_float line.len ;
  print_string "\n" ;
  print_string "Adjacent sector : " ;
  (match line.adjacentsector with -1 -> print_string "NONE\n"
    | i -> print_string "PRESENT = "; print_int i ; print_string "\n") ;
  print_string "MainTx : " ;
  print_int line.maintx ;
  print_string "\n" ;
  print_string "UpperTx : " ;
  print_int line.uppertx ;
  print_string "\n" ;
  print_string "LowerTx : " ;
  print_int line.lowertx ;
  print_string "\n" ;
  print_string "Passable : " ;
  print_string (string_of_bool line.passable) ;
  print_string "\n" ;
  Format.print_flush ()
;;


let printSector s_id =
 let sect = globalsectortable.(s_id) in
  print_string "Sector #" ; print_int s_id ;
  print_string " bounds " ;
  List.iter (fun i -> print_int i ; print_string " ") sect.bounds ;
  print_string "\n" ;
  print_float sect.ceilingh ;
  print_string "\n" ;
  print_float sect.floorh ;
  print_string "\n" ;
  Format.print_flush ()
;;



let printTree tree =
 let rec recPrintTree indent = function
    Leaf lopt ->
              begin
              match lopt with
                 [] ->
                  print_string (String.make indent ' ') ;
                  print_string "NULL LEAF\n"
               | l ->
                  print_string (String.make indent ' ') ;
                  print_string "LEAF : [ " ;
                  List.iter (fun i -> print_int i ; print_string " ") l ;
                  print_string "]\n"
              end
  | Node n -> print_string (String.make indent ' ') ;
              print_string "NODE Partition line : " ;
              print_int n.partline ;
              print_string "\n" ;
              print_string (String.make indent ' ') ;
              print_string "Left bounding box " ;
              (match n.leftbbox with
                 Some (leftxmin,leftymin,leftxmax,leftymax) ->
                   print_float leftxmin ; print_string " " ;
                   print_float leftymin ; print_string " " ;
                   print_float leftxmax ; print_string " " ;
                   print_float leftymax ; print_string "\n" ;
               | None -> print_string "No bounding box for left\n") ;
              print_string (String.make indent ' ') ;
              print_string "Left :\n" ;
              recPrintTree (indent+1) n.left ;
              print_string (String.make indent ' ') ;
              print_string "Right bounding box " ;
              (match n.rightbbox with
                 Some (rightxmin,rightymin,rightxmax,rightymax) ->
                   print_float rightxmin ; print_string " " ;
                   print_float rightymin ; print_string " " ;
                   print_float rightxmax ; print_string " " ;
                   print_float rightymax ; print_string "\n" ;
                | None ->print_string "No bounding box for right\n") ;
               print_string (String.make indent ' ') ;
               print_string "Right :\n" ;
               recPrintTree (indent+1) n.right
  in
   recPrintTree 0 tree ;
   Format.print_flush ()
;;



(* IMPRESSION D'UNE BLOCKMAP *)
let printBlockMap nb_horiz_block nb_vert_block vert_ptr =
 print_string "BlockMap size : " ;
 print_int nb_horiz_block ;
 print_string " horizontal blocks and " ;
 print_int nb_vert_block ;
 print_string " vertical blocks" ;
 print_string "\n" ;
 for i = 0 to nb_vert_block - 1
  do
  for j = 0 to nb_horiz_block - 1
   do
   List.iter (fun x -> print_int x ; print_string " ")
                                (vert_ptr.(i)).(j) ;
   print_string "| "
   done ;
  print_string "\n"
  done ;
 Format.print_flush ()
;;



let printAllData bsp_tree =
 for i = 0 to !nbVertexes-1
  do printVertex i done ;
 for i = 0 to !nbLinedefs-1
  do printLinedef i done ;
 for i = 0 to !nbSectors-1
  do printSector i done ;
 printTree bsp_tree
;;
