open Renderextern ;;
open Vertexsbase ;;
open Linedefsbase ;;
open Sectorsbase ;;
open Bsptreebase ;;
open Svgalib ;;
open Location ;;
open Angleextra ;;


(* Offset horizontal dans la texture mappee (No. colonne) *)
let hIndex = ref 0 ;;

(* Distance du pt de vue a la projection orthogonale sur le mur *)
let pDist = ref 0.0 ;;

(* Distance du point de projection orthogonale au debut du mur *)
let bToBP = ref 0.0 ;;

(* Angle du mur courant *)
let line_angle = ref 0.0 ;;

(* Offset vertical dans la texture (No de ligne) *)
let vIndex = ref 0.0 ;;

(* Increment pour avoir les offsets verticaux successifs *)
let vIncr = ref 0.0 ;;

(* Profondeur minimale en deca de laquelle on considere que l'on  *)
(* ne voit pas un objet ou un mur. Ceci permet de clipper au plan *)
(* profondeur = ZMIN, et donc on ne risque plus de division par 0 *)
(* lors du calcul de InvDistance ().                              *)
(* let zMIN = 1.0 ;; *)

(* Couleur actuelle de dessin (sert pour sol et plafond uniquement) *)
let drawing_color = ref (Char.chr 0) ;;

let first_drawn_sector = ref false ;;





(* **************************************************************** *)
(* Fonction de trace de ligne vertical dans la colonne ColumnIndex. *)
(* La ligne s'etend verticalement entre RealTop et RealBottom.      *)
(* Cette fonction ne se charge pas de faire le clipping.            *)
(* Version sans texturage.                                          *)
(* **************************************************************** *)
let vertical_line_draw columnIndex realTop realBottom =
 (* Calcul de l'index de depart dans la memoire video *)
 let indexStart = (realTop * 320 + columnIndex)
 (* ... et de l'index de fin *)
 and indexEnd = (realBottom * 320 + columnIndex) in
 let index = ref indexStart in
 while !index < indexEnd do
   String.unsafe_set dblBuffer !index !drawing_color ;
   index := !index + 320
 done
;;




(* **************************************************************** *)
(* Fonction de trace de ligne vertical dans la colonne ColumnIndex. *)
(* La ligne s'etend verticalement entre RealTop et RealBottom.      *)
(* Cette fonction ne se charge pas de faire le clipping.            *)
(* Version avec texturage.                                          *)
(* **************************************************************** *)
(*
let vertical_textured_line_draw columnIndex realTop realBottom =
  (* Calcul de l'index de depart dans la memoire video *)
  let indexStart = (realTop * 320 + columnIndex)
  (* ... et de l'index de fin *)
  and indexEnd = (realBottom * 320 + columnIndex) in
  let current_bitmap = !current_bitmap in
  let current_bitmap_height = !current_bitmap_height in
  let current_bitmap_width = !current_bitmap_width in
  let hIndex = !hIndex in
  let vIncr = !vIncr in
  let rec draw index vi =
    if index < indexEnd then begin
      let color = current_bitmap.(((truncate vi) mod current_bitmap_height)
                    * current_bitmap_width + hIndex) in
      String.unsafe_set dblBuffer index color ;
      draw (index + 320) (vi +. vIncr)
    end in
  draw indexStart !vIndex
;;
*)
let vertical_textured_line_draw columnIndex realTop realBottom =
  (* Calcul de l'index de depart dans la memoire video *)
  let indexStart = (realTop * 320 + columnIndex)
  (* ... et de l'index de fin *)
  and indexEnd = (realBottom * 320 + columnIndex) in
  let current_bitmap = !current_bitmap in
  let current_bitmap_height = !current_bitmap_height in
  let current_bitmap_width = !current_bitmap_width in
  let hIndex = !hIndex in
  let vIncr = truncate (!vIncr *. 256.0) in
  let vIndex = truncate (!vIndex *. 256.0) in
  let rec draw index vi =
    if index < indexEnd then
      begin
      let color = current_bitmap.(((vi asr 8) mod current_bitmap_height)
                    * current_bitmap_width + hIndex) in
      String.unsafe_set dblBuffer index color ;
      draw (index + 320) (vi + vIncr)
      end in
  draw indexStart vIndex
;;


let add_wall newXStart newXEnd screen_x_start screen_x_end rt rb =
  (* On calcule les coordonnees verticales de debut et de fin *)
  (* de chacune des extremites du mur projete sur l'ecran     *)
  let swap0 =invDistanceTable.(truncate newXStart)
  and swap1 = invDistanceTable.(truncate newXEnd) in
  let screen_top_start = 100 + (truncate (rt *. swap0))
  and screen_bottom_start = 100 + (truncate (rb *. swap0))
  and screen_top_end = 100 + (truncate (rt *. swap1))
  and screen_bottom_end = 100 + (truncate (rb *. swap1)) in
  (* Initialisation des valeurs par defaut des extremites *)
  (* verticales du mur courant.                           *)
  let top = ref (float screen_top_start)
  and bottom = ref (float screen_bottom_start)
  and swap2 = (float (screen_x_end - screen_x_start)) in
  (* Deltas pour calculer incrementalement les hauteurs du mur *)
  let delta_top = (float (screen_top_end - screen_top_start)) /. swap2 in
  let delta_bottom = (float (screen_bottom_end - screen_bottom_start))
                    /. swap2 in
  (* Maintenant, on fait les clipping a la taille horizontale de l'ecran *)
  let screen_x_end_clipped = ref screen_x_end
  and screen_x_start_clipped = ref screen_x_start
  (* Et on prepare les variables clipped_top et clipped_bottom *)
  and clipped_top = ref 0 and clipped_bottom = ref 0 in
  if screen_x_end > 319 then screen_x_end_clipped := 319 ;
  if screen_x_start < 0 then
    (top := !top -. ((float screen_x_start) *. delta_top) ;
     bottom := !bottom -. ((float screen_x_start) *. delta_bottom) ;
     screen_x_start_clipped := 0) ;
  (* Pour toutes les colonnes que le mur occupe (sauf recouvrement) *)
  (* a l'ecran, on va dessiner les "tranches verticales".           *)
  for columnIndex = !screen_x_start_clipped to !screen_x_end_clipped do
    if columnDone.(columnIndex) <> TXFLAG_MAIN then
      begin
      (* On ne dessine que si cette colonne de l'ecran n'est *)
      (* pas deja occupee par un mur qui etait plus proche.  *)

      (* En fait, ces calculs ne sont utiles que quand on *)
      (* dessine un maintx. Il faudrait peut etre eviter  *)
      (* de les faire quand ca n'est pas utile.           *)

      (* Optim: 90 DEG = 1.5707963268 RAD *)
      let tanIndex = !line_angle +. 1.5707963268 -. angleTable.(columnIndex) in
      hIndex := (truncate ((!pDist *. (tan tanIndex)) +. !bToBP))
                mod !current_bitmap_width ;
      (* On contre-carre les erreurs d'arrondis *)
      if (!hIndex < 0) then hIndex := 0 ;

      (* Sinon, ca ne doit pas gener car un delta infini ne peut *)
      (* pas etre utilise quelque part, ca ne serait pas logique *)
      (* enfin, esperons...                                      *)
      if (!bottom <> !top) then vIncr := (rb -. rt) /. (!bottom -. !top) ;

      (* On clippe en haut *)
      clipped_top := max (truncate !top) topColumn.(columnIndex) ;
      (* On clippe en bas *)
      clipped_bottom := min (truncate !bottom) bottomColumn.(columnIndex) ;

      (* Index vertical apres clipping. N'est utile que si  *)
      (* on dessine un maintx. On devrait peut etre virer   *)
      (* ca pour gagner du temps quand justement on dessine *)
      (* autrechose qu'un maintx...                         *)
      vIndex := ((float !clipped_top) -. !top) *. !vIncr ;

      (* Pour contrecarrer les erreurs d'arrondi *)
      if (!vIndex < 0.0) then vIndex := 0.0 ;

      (* Inutile de s'assurer que l'index reste bien verticalement   *)
      (* dans la bitmap car on le fait a chaque fois qu'on l'utilise *)
      (* lors du dessin des point, donc ce sera toujours fait...     *)

      (* On dessine le mur *)
      (* Mur en lui meme est texture *)
      vertical_textured_line_draw columnIndex !clipped_top !clipped_bottom ;

      (* Le plafond n'est pas texture *)
      (* Pour eviter que le plafond n'entre dans un lowertx *)
      (* il faut eviter que le haut du plafond ne soit plus *)
      (* bas que le haut du plancher. C'est pour cela que   *)
      (* l'on clippe clipped_top si necessaire.                 *)
      drawing_color := Char.chr 10 ;
      if (!txFlag = TXFLAG_UPPER) || (!txFlag = TXFLAG_MAIN) then
        (if !clipped_top > bottomColumn.(columnIndex)
         then clipped_top := bottomColumn.(columnIndex) ;
         vertical_line_draw columnIndex topColumn.(columnIndex) !clipped_top ;
         topColumn.(columnIndex) <-
	            max topColumn.(columnIndex) !clipped_bottom) ;

      (* Le plancher n'est pas non plus texture. *)
      (* Il faut eviter que le bas du plancher ne soit plus     *)
      (* haut que le bas du plafond sinon le plancher entrerait *)
      (* dans un uppertx. Donc on clippe aussi clipped_bottom si    *)
      (* necessaire.                                            *)
      drawing_color := Char.chr 6 ;
      if (!txFlag = TXFLAG_LOWER) || (!txFlag = TXFLAG_MAIN) then
        (if !clipped_bottom < topColumn.(columnIndex)
         then clipped_bottom := topColumn.(columnIndex) ;
         vertical_line_draw columnIndex
	                    !clipped_bottom bottomColumn.(columnIndex) ;
	 bottomColumn.(columnIndex) <-
	                min bottomColumn.(columnIndex) !clipped_top) ;

      (* Maintenant, la colonne est ocuppee par un mur si et seulement *)
      (* si on etait en train de dessiner un maintx                    *)
      columnDone.(columnIndex) <- !txFlag ;
      (* On a fait une colonne de plus si elle est complete *)
      if !txFlag = TXFLAG_MAIN then nbColumnsDone := !nbColumnsDone + 1
      end ;

    (* On calcule incrementalement les extremites *)
    (* verticales de la prochaine tranche de mur. *)
    top := !top +. delta_top ;
    bottom := !bottom +. delta_bottom
  done
;;





(* ********************************************************************* *)
(* Premiere fonction principale de rendu. Elle prend en parametre une    *)
(* linedef, et l'affiche si elle bien visible et effectue les clippings  *)
(* la projection (calcul de projection) et les clipping horizontaux.     *)
(* Elle initie aussi le mapping de la texture en pre-calculant certaines *)
(* valeurs. Elle determine les parties potentiellement visibles du mur   *)
(* en fonction des main_tx, upper_tx, et lower_tx. En fonction de ca,    *)
(* elle delegue ses pouvoirs a la fonction chargee de rajouter les murs  *)
(* a l'ecran.                                                            *)
(* ********************************************************************* *)
let draw_linedef linedef =
 (* On calcule les nouvelles coordonnees du segment. Pour cela, on *)
 (* translate le repere au niveau du joueur et on le fait tourner  *)
 (* de facon a ce que la direction de vue du joueur coincide avec  *)
 (* l'angle 0.                                                     *)
  let swapangle = toRad(-. !playerAngle) in
  let swapcos = cos (swapangle)
  and swapsin = sin (swapangle) in
  let newXStart = ref (((globalvertextable.(linedef.src)).x -. !playerX)
		       *. swapcos -. ((globalvertextable.(linedef.src)).y
		       -. !playerY) *. swapsin)
  and newXEnd = ref (((globalvertextable.(linedef.dest)).x -. !playerX)
		     *. swapcos -. ((globalvertextable.(linedef.dest)).y
		     -. !playerY) *. swapsin)
  and newYStart = ref (((globalvertextable.(linedef.src)).y -. !playerY)
		       *. swapcos +. ((globalvertextable.(linedef.src)).x
		       -. !playerX) *. swapsin)
  and newYEnd = ref (((globalvertextable.(linedef.dest)).y -. !playerY)
		     *. swapcos +. ((globalvertextable.(linedef.dest)).x
		     -. !playerX) *. swapsin) in
  if (!newXStart < 1.0) && (!newXEnd < 1.0)
  then () (* Le mur est totalement derriere nous, donc on laisse *)
  else
   begin
   (* Pre-requis pour le mapping et coefficients intermediaires *)
   (* stockes en variables globales pour gagner un peu de temps *)
   let l = linedef.len in
   let r = ((!newYStart) *. (!newYStart -. !newYEnd)-.
            (!newXStart) *. (!newXEnd -. !newXStart)) /. (l*.l)
   and s = ((!newYStart) *. (!newXEnd -. !newXStart)-.
	    (!newXStart) *. (!newYEnd -. !newYStart)) /. (l*.l) in
   line_angle := linedef.angle +. swapangle ;
   (* Distance du point (0,0) (oeil) a la droite representant le mur *)
   pDist := s *. l ;
   (* Distance du la projection ortho de l'oeil sur le mur au debut du mur *)
   bToBP := r *. l ;
   (* On va faire les clippings pour si l'un des x est derriere nous. *)
   if !newXStart < 1.0
    then (newYStart := !newYStart +. (1.0 -. !newXStart) *. tan (!line_angle) ;
          newXStart := 1.0) ;
   if !newXEnd < 1.0
    then (newYEnd := !newYEnd +. (1.0 -. !newXEnd) *. tan (!line_angle) ;
          newXEnd := 1.0) ;
   (* Maintenant, on clippe la profondeur (correspondant sur notre plan   *)
   (* a l'abscisse) de nos 2 points de facon a ce qu'au bout d'un moment, *)
   (* tous les murs trop eloignes soient quand meme de la meme hauteur.   *)
   (* Ca permet au passage d'utiliser une table precalculee plutot que de *)
   (* lancer une fonction de calcul a chaque fois.                        *)
   if (!newXStart > 9999.0) then newXStart := 9999.0 ;
   if (!newXEnd > 9999.0) then newXEnd := 9999.0 ;
   (* On calcule la projection en x <sur l'ecran> des extremites du murs *)
   (* Autrement dit, on obtient les numeros de colonnes d'ou part et ou   *)
   (* arrive le mur.                                                      *)
   let screen_x_start = 160 - (truncate
		   ((!newYStart) *. invDistanceTable.(truncate !newXStart)))
   and screen_x_end = 160 - (truncate
       		 ((!newYEnd) *. invDistanceTable.(truncate !newXEnd))) in
   (* On verifie que le mur n'est pas totalement invisible *)
   if (screen_x_start = screen_x_end) || (screen_x_end < 0)
      || (screen_x_start > 319)
   then () (* Le mur est invisible alors on s'en va *)
   else
    begin
    (* On met a jour la hauteur du joueur si necessaire, c'est a dire    *)
    (* si on est sur la premiere linedef a dessiner. En effet, dans ce   *)
    (* cas, son altitude basse (etant celle de son secteur de reference) *)
    (* definit la nouvelle altitude du joueur.                           *)
    if !first_drawn_sector then
      (first_drawn_sector := false;
       playerH := !playerH +. (globalsectortable.(linedef.sector)).floorh) ;

    (* On se tappe les coeffintermediaires de hauteur des yeux du joueur. *)
    if linedef.maintx <> 255
    then
     begin
     let rt = !playerH -. (globalsectortable.(linedef.sector)).ceilingh
     and rb = !playerH -. (globalsectortable.(linedef.sector)).floorh in
     txFlag := TXFLAG_MAIN ;
     current_bitmap := (bitmaps_table.(linedef.maintx)).txData ;
     current_bitmap_width := (bitmaps_table.(linedef.maintx)).txWidth ;
     current_bitmap_height := (bitmaps_table.(linedef.maintx)).txHeight ;
     add_wall !newXStart !newXEnd screen_x_start screen_x_end rt rb
     end
    else
     begin
     (let rb =
       (if linedef.uppertx <> 255 then
         begin
         current_bitmap := (bitmaps_table.(linedef.uppertx)).txData ;
         current_bitmap_width := (bitmaps_table.(linedef.uppertx)).txWidth ;
         current_bitmap_height := (bitmaps_table.(linedef.uppertx)).txHeight ;
         !playerH -. (globalsectortable.(linedef.adjacentsector)).ceilingh
         end
        else !playerH -. (globalsectortable.(linedef.sector)).ceilingh)
     and rt = !playerH -. (globalsectortable.(linedef.sector).ceilingh) in
     txFlag := TXFLAG_UPPER ;
     add_wall !newXStart !newXEnd screen_x_start screen_x_end rt rb) ;

     (let rt =
       (if linedef.lowertx <> 255 then
         begin
         current_bitmap := (bitmaps_table.(linedef.lowertx)).txData ;
         current_bitmap_width := (bitmaps_table.(linedef.lowertx)).txWidth ;
         current_bitmap_height := (bitmaps_table.(linedef.lowertx)).txHeight ;
         !playerH -. (globalsectortable.(linedef.adjacentsector)).floorh
         end
        else !playerH -. (globalsectortable.(linedef.sector)).floorh)
     and rb = !playerH -. (globalsectortable.(linedef.sector).floorh) in
     txFlag := TXFLAG_LOWER ;
     add_wall !newXStart !newXEnd screen_x_start screen_x_end rt rb)
     end
    end
   end
;;




let rec draw_leaf = function
  | [] -> ()
  | h::q -> draw_linedef (globallinedeftable.(h)) ; draw_leaf q
;;


let is_right_visible (x1, y1, x2, y2) =
  if (!playerLeftAngle < 90.0)
  then (!playerX <= x2)
  else
    if (!playerLeftAngle < 180.0)
    then (!playerY < y2)
    else
      if (!playerLeftAngle < 270.0)
      then (!playerX >= x1)
      else (!playerY >= y1)
;;


let is_left_visible = is_right_visible ;;



(* ****************************************************** *)
(* FONCTION DE PARCOURS DU BSP TREE. ELLE DESSINE LA VUE. *)
(* C'EST UN PARCOURS OU L'ON RECOIT LES MURS LES PLUS     *)
(* PROCHES DU JOUEUR EN PREMIER.                          *)
(* ****************************************************** *)
let rec front_to_back_tree_parsing ourTree =
  (* ON VERIFIE QUE L'ON N'A PAS DEJA REMPLI L'ECRAN *)
  if !nbColumnsDone < 320 then
   begin
   match ourTree with
    | Leaf lineid_lst -> draw_leaf lineid_lst  (* ON DESSINE LA FEUILLE *)
    | Node nd ->
	let v1 = globalvertextable.((globallinedeftable.(nd.partline)).src)
	and v2 = globalvertextable.((globallinedeftable.(nd.partline)).dest) in
	(* ON CLASSIFIE LE JOUEUR PAR RAPPORT A LA LIGNE *)
	(* DE PARTITIONNEMENT ACTUELLE.                  *)
	match getPointPosition (!playerX, !playerY)
	                       (v1.x, v1.y) (v2.x, v2.y) with
	 | P_Left ->
	     (* LE JOUEUR EST A GAUCHE, ON VA DONC A GAUCHE EN PREMIER    *)
	     (* L'ARBRE EST MOITIE BIEN STRUCTURE. SI LA BOUNDING BOX     *)
	     (* EST NONE ALORS C'EST QUE LA FEUILLE SUIVANTE CONTIENT UNE *)
	     (* LISTE VIDE DE FEUILLES DONC QU'ON N'A PAS BESOIN D'ALLER  *)
	     (* Y VOIR. BON, C'EST UN PEU CRAD, MAIS BON... A REVOIR !    *)
	     (match nd.leftbbox with
               | None -> ()
               | Some bb ->
		   if is_left_visible bb
		    then front_to_back_tree_parsing nd.left) ;
	     draw_linedef (globallinedeftable.(nd.partline)) ;
	     (match nd.rightbbox with
               | None -> ()
               | Some bb ->
		   if is_right_visible bb
		    then front_to_back_tree_parsing nd.right)
	 | P_Right ->
	     (* LE JOUEUR EST A DROITE, ON VA DONC A DROITE EN PREMIER *)
	     (match nd.rightbbox with
               | None -> ()
               | Some bb ->
		   if is_right_visible bb
		    then front_to_back_tree_parsing nd.right) ;
	     draw_linedef (globallinedeftable.(nd.partline)) ;
	     (match nd.leftbbox with
               | None -> ()
               | Some bb ->
		   if is_left_visible bb
		    then front_to_back_tree_parsing nd.left)
    | P_On -> draw_linedef (globallinedeftable.(nd.partline)) ;
 end
;;




(* ************************************************************* *)
(* FONCTION DE RENDU. C'EST ELLE QUI APPELLE LA PARCOURS DU BSP. *)
(* ************************************************************* *)
let render_view () =
  (* ON NOTIFIE QU'ON N'A DESSINNE AUCUNE COLONNE. *)
  nbColumnsDone := 0 ;
  for i=0 to 319 do
    topColumn.(i) <- 0 ;
    bottomColumn.(i) <- 200 ;
    columnDone.(i) <- TXFLAG_UNDEF ;
  done ;
  (* ON DIT QUE LE PROCHAIN SECTEUR QUE L'ON VA DESSINER SERA LE PREMIER *)
  first_drawn_sector := true ;
  (* TAILLE DU JOUEUR *)
  playerH := 60.0 ;
  (* ET ON PARCOURS LE BSP AFIN DE DESSINER LA VUE. *)
  front_to_back_tree_parsing (!levelBspTree) ;
  (* TRANSFERT DE NOTRE DOUBLE-BUFFER EN MEMOIRE VIDEO *)
  vga_blit_buffer dblBuffer
;;
